'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var view = require('@codemirror/view');
var language = require('@codemirror/language');

function colorPickersDecorations(view$1) {
    const widgets = [];
    for (const range of view$1.visibleRanges) {
        language.syntaxTree(view$1.state).iterate(Object.assign(Object.assign({}, range), { enter: (type, from, to) => {
                if (type.name === 'ColorLiteral') {
                    const color = view$1.state.doc.sliceString(from, to);
                    const d = view.Decoration.widget({
                        widget: new ColorPickerWidget(color),
                        side: 1,
                    });
                    widgets.push(d.range(to));
                }
            } }));
    }
    return view.Decoration.set(widgets);
}
function toRRGGBBHex(color) {
    if (color.startsWith('#')) {
        if (color.length === 7) {
            return color;
        }
        if (color.length === 4) {
            // Convert #rgb to #rrggbb
            return `#${color[1].repeat(2)}${color[2].repeat(2)}${color[3].repeat(2)}`;
        }
    }
    return color;
}
const wrapperClassName = 'cm-css-color-picker-wrapper';
class ColorPickerWidget extends view.WidgetType {
    constructor(color) {
        super();
        this.color = color;
    }
    eq(other) {
        return other.color === this.color;
    }
    toDOM() {
        const picker = document.createElement('input');
        picker.type = 'color';
        // Color picker only accepts long hex
        picker.value = toRRGGBBHex(this.color);
        const wrapper = document.createElement('span');
        wrapper.appendChild(picker);
        wrapper.className = wrapperClassName;
        return wrapper;
    }
    ignoreEvent() {
        return false;
    }
}
const colorPicker = view.ViewPlugin.fromClass(class {
    constructor(view) {
        this.decorations = colorPickersDecorations(view);
    }
    update(update) {
        if (update.docChanged || update.viewportChanged) {
            this.decorations = colorPickersDecorations(update.view);
        }
    }
}, {
    decorations: (v) => v.decorations,
    eventHandlers: {
        change: (e, view) => {
            const target = e.target;
            if (target.nodeName !== 'INPUT' ||
                !target.parentElement ||
                !target.parentElement.classList.contains(wrapperClassName)) {
                return false;
            }
            const pos = view.posAtDOM(target);
            const strAtPos = view.state.doc.sliceString(Math.max(0, pos - 7), pos);
            const currentColor = strAtPos.slice(strAtPos.indexOf('#'));
            if (!currentColor) {
                return false;
            }
            view.dispatch({
                changes: {
                    from: pos - currentColor.length,
                    to: pos,
                    insert: target.value,
                },
            });
            return true;
        },
    },
});

exports.colorPicker = colorPicker;
exports.wrapperClassName = wrapperClassName;
